@@ -1,4 +1,4 @@
-/* $Id: conf-osf1.h,v 1.2 1993/08/19 05:26:59 genek Exp $ */
+/* $Id: conf-osf1.h,v 1.3 1994/07/15 11:02:47 gkim Exp $ */
 
 /*
  * conf-osf1.h
@@ -36,7 +36,7 @@
  * does your system have a <stdlib.h> like POSIX says you should? 
  */
 
-#undef STDLIBH
+#define STDLIBH
 
 /* 
  * does your system have gethostname(2) (instead of uname(2))?
