@@ -1,4 +1,4 @@
-/* $Id: conf-xenix.h,v 1.1.1.3 92/11/03 15:39:45 genek Exp $ */
+/* $Id: conf-xenix.h,v 1.3 1993/08/19 05:27:10 genek Exp $ */
 
 /*
  * conf-xenix.h
@@ -56,8 +56,15 @@
 
 #undef GETHOSTNAME
 
+/*
+ * To work around lack of gettimeofday() in src/dbase.build.c
+ */
+
+#define NOGETTIMEOFDAY
+
 /*
  * miscallaneous stuff 
  */
 
 #define XENIX
+
