@@ -1,32 +1,32 @@
-/* $Id: list.h,v 1.2 1993/08/19 05:27:27 genek Exp $ */
+/* $Id: list.h,v 1.5 1994/07/15 11:02:53 gkim Exp $ */
 
-#define LIST_HASHSZ	17011
+/* #define LIST_HASHSZ	17011 */
+#define LIST_HASHSZ	6007		/* a more reasonable hash size */
 
-/* linked list element */
+/* Thanks to Paul Hilchey for cleaning this up */
+
+/* data node: these get linked into both a doubly linked list (using next
+   and prev) and a chained hash table (using cnext) */
 struct list_elem {
     char 	*varname;
     char	*varvalue;
     int		priority;
-    struct list_elem	*next;
-    struct list_elem	*prev;
     int		flag;
+    struct list_elem	*next;		/* pointer to next entry on list */
+    struct list_elem	*prev;		/* pointer to prev entry on list */
+    struct list_elem	*cnext;		/* pointer to next entry on chain */
 };
 
-/* hash table structure */
-struct list_hash {
-    char	*key;			/* hashed string */
-    int		used;			/* table used or not */
-    struct list_elem	*lptr;		/* pointer to entry on list */
-    struct list_hash	*next;		/* pointer to next entry on chain */
-};
-
-/* linked list pointer, with head and tail */
+/* hash list: NULL pointers are used instead of list/chain sentinels */
 struct list {
-    struct list_elem	*p_head;
-    struct list_elem	*p_tail;
-    struct list_hash	hashtable[LIST_HASHSZ];
+    struct list_elem	*p_head;	/* head of linked list */
+    struct list_elem	*p_tail;	/* tail of linked list */
+    struct list_elem	*p_curr;	/* current element for list traversal */
+    struct list_elem	*hashtable[LIST_HASHSZ];	/* hash chains */
 };
 
+typedef struct list list;
+
 /* prototypes */
 /* Do not remove this line.  Protyping depends on it! */
 #if defined(__STDC__) || defined(__cplusplus)
