@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "$Id: crc.c,v 1.4 1993/12/15 16:50:07 genek Exp $";
+static char rcsid[] = "$Id: crc.c,v 1.6 1994/07/25 15:46:27 gkim Exp $";
 #endif
 
 /*
@@ -118,6 +118,7 @@ UpdateCRC16(buffer, bufsize)
     }
 }
 
+int
 sig_crc_get(fd_in, ps_signature, siglen)
     int fd_in;
     char *ps_signature;
@@ -143,7 +144,7 @@ sig_crc_get(fd_in, ps_signature, siglen)
     if (printhex)
 	sprintf(ps_signature, "%04lx", (crc16 &0xFFFF));
     else {
-	unsigned long l = (unsigned long)(crc16 & 0xFFFF);
+	uint32 l = (uint32)(crc16 & 0xFFFF);
 	sprintf(ps_signature, "%s", pltob64(&l, (char *)scratch, 1));
     }
 
