@@ -6,10 +6,14 @@
  ** Revised (for MD5): RLR 4/27/91                                    **
  **********************************************************************/
 
-/* $Id: md5.h,v 1.4 1993/11/22 06:38:20 genek Exp $ */
+/* $Id: md5.h,v 1.5 1994/07/15 11:03:26 gkim Exp $ */
 
 /* typedef a 32-bit type */
+#ifdef TW_TYPE32
+typedef unsigned TW_TYPE32 UINT4;
+#else
 typedef unsigned long int UINT4;
+#endif
 
 /* Data structure for MD5 (Message-Digest) computation */
 typedef struct {
