@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "$Id: config.parse.c,v 1.17 1993/12/13 04:02:34 genek Exp $";
+static char rcsid[] = "$Id: config.parse.c,v 1.21 1994/07/21 01:03:26 gkim Exp $";
 #endif
 
 /*
@@ -81,7 +81,7 @@ configfile_read(pp_list, pp_entry_list)
     struct list **pp_list;
     struct list **pp_entry_list;
 {
-    FILE 	*fpin, *fpout;
+    FILE 	*fpin, *fpout = (FILE *) NULL;
     char	filename[MAXPATHLEN+512];
     char	ignorestring[1024];
     char	s[MAXPATHLEN+1024];
@@ -99,7 +99,7 @@ configfile_read(pp_list, pp_entry_list)
 	fputs("### Phase 1:   Reading configuration file\n", stderr);
 
     /* generate temporary file name */
-    if ((tmpfilename = (char *) malloc(L_tmpnam)) == NULL) {
+    if ((tmpfilename = (char *) malloc(L_tmpnam + MAXPATHLEN)) == NULL) {
 	perror("configfile_read: malloc()");
 	exit(1);
     };
@@ -111,26 +111,26 @@ configfile_read(pp_list, pp_entry_list)
     }
 
     /* generate configuration file name */
-    if (specified_configfile == NULL)
+    if (specified_configmode != SPECIFIED_FILE)
 	sprintf(configfile, "%s/%s", config_path, config_file);
     else
 	(void) strcpy(configfile, specified_configfile);
 
     /* open the files */
-    /*		check to see if input is just stdin */
-    if (*configfile == '-' && !configfile[1]) {  /* configfile == "-" */
-	fpin = stdin;
-    } 
     /* were we given a specific fd to use? */
-    else if (specified_configfd != -1) {
+    if (specified_configmode) {
 	char errstr[1024];
 
 	/* we already checked to see that it's a valid fd */
-	if (!(fpin = fdopen(specified_configfd, "r"))) {
+	if (!(fpin = (FILE *) fdopen(specified_configfd, "r"))) {
 	    sprintf(errstr, "tripwire: Couldn't open fd!  fdopen()");
 	    perror(errstr);
 	    exit(1);
 	}
+	rewind(fpin);
+	if ((err = ftell(fpin)) != 0) {
+	    die_with_err("configfile_read: ftell()", NULL);
+	}
     }
     /* otherwise, it's just a normal file */
     else if ((fpin = fopen(configfile, "r")) == NULL) {
@@ -140,6 +140,13 @@ configfile_read(pp_list, pp_entry_list)
 	exit(1);
     }
 
+    /* if we've already preprocessed the file, we can skip this */
+    /*		uh, we used to only parse the tw.config file once.
+     *		this causes problems during interactive updates.
+     *		so, with v1.2, we're a little slower, so sue me.
+     * -ghk
+     */
+
     err = umask(077);  /* to protect the tempfile */
 
     if ((fpout = fopen(tmpfilename, "w+")) == NULL) {
@@ -169,24 +176,15 @@ configfile_read(pp_list, pp_entry_list)
 
     tw_macro_parse(configfile, fpin, fpout, (struct list **) pp_entry_list);
 
-    if (fpin != stdin)
+    if (!specified_configmode)
 	(void) fclose(fpin);
-#ifdef FOO
-    if (fflush(fpout) == EOF) {
-	char s[256];
-	/*
-        fputs("configfile_read: unknown error on fflush(fpout)\n", stderr);
-	*/
-        sprintf(s, "configfile_read: fflush(fpout)");
-	perror(s);
-	exit(1);
-    }
-    else
-#endif
+
     fflush(fpout);
     rewind(fpout);
-
     fpin = fpout;
+    if ((err = ftell(fpin)) != 0) {
+	die_with_err("configfile_read: ftell()", NULL);
+    }
 
     /* do we just print out the file, and then exit? */
     if (printpreprocess) {
@@ -197,6 +195,9 @@ configfile_read(pp_list, pp_entry_list)
 	exit(0);	
     }
 
+ALREADY_PREPROCESSED:			/* LABEL */
+	;
+
     /* pass 1: get all of the prune entries '!' */
     while (fgets(s, sizeof(s), fpin) != NULL) {
 
@@ -246,6 +247,7 @@ configfile_read(pp_list, pp_entry_list)
 
 	/* add to local prune list */
 	list_set(filename, "", 0, &prune_list);
+	SPDEBUG(1000) printf("configfile_read: pruning %s\n", filename);
 
 	/* set appropriate prune flag */
 	list_setflag(filename, prune_mode, &prune_list);
@@ -307,6 +309,9 @@ configfile_read(pp_list, pp_entry_list)
 	    continue;
 	}
 
+	/* expand any escaped octal characters in name */
+	filename_escape_expand(filename);
+
 	/*
 	 * convert configuration-file style ignore-string to our database
 	 * representation.
@@ -316,6 +321,9 @@ configfile_read(pp_list, pp_entry_list)
 	/* add it to the list while it still has escaped characters */
 	sprintf(number, "%d %s", entrynum, ignorestring);
 	list_set(filename, number, 0, pp_entry_list);
+	SPDEBUG(1000) 
+	printf("configfile_read: adding %s (%d)\n", filename, number);
+
 	/* reverse index (number --> entryname) */
 	{
 	    char eindex[50];
@@ -324,9 +332,6 @@ configfile_read(pp_list, pp_entry_list)
 	    list_setflag(eindex, 1, pp_entry_list);
 	}
 
-	/* expand any escaped octal characters in name */
-	filename_escape_expand(filename);
-
 	/* pass down the priority -- based on how fully-qualified the
 	 * 	entry was.
 	 */
@@ -345,8 +350,10 @@ configfile_read(pp_list, pp_entry_list)
 	list_print(pp_list);
 
     /* clean up */
-    (void) fclose(fpin);
+    if (!specified_configmode)
+	(void) fclose(fpin);
 
+    rewind(fpout);
     return;
 }
 
@@ -433,8 +440,27 @@ printf("configfile_descend: r=%d: %s\n", countrecurse, filename);
 	majordev = major(statbuf.st_dev);
 	minordev = minor(statbuf.st_dev);
     } else {
+#ifdef apollo
+	/*
+	 * It seems that Apollos have (3904,0) for dirs, (3904,1) for files.
+	 * So how do we prevent ourselves from crossing filesystems
+	 * (descending into mounted disks)?
+	*/
+	if ((major(statbuf.st_dev) != majordev ||
+		minor(statbuf.st_dev) != minordev) &&
+		! ( majordev == 3904 &&
+		    minordev == 0 &&
+		    major(statbuf.st_dev) == 3904 &&
+		    minor(statbuf.st_dev) == 1 ) &&
+		! ( majordev == 3904 &&
+		    minordev == 1 &&
+		    major(statbuf.st_dev) == 3904 &&
+		    minor(statbuf.st_dev) == 0 ))
+#else
 	if (major(statbuf.st_dev) != majordev ||
-					minor(statbuf.st_dev) != minordev) {
+		minor(statbuf.st_dev) != minordev)
+#endif
+	{
 
 SPDEBUG(4)
 printf("configfile_descend: pruning '%s' n(%d,%d) != o(%d, %d)\n", filename,
@@ -491,8 +517,21 @@ printf("configfile_descend: pruning '%s' n(%d,%d) != o(%d, %d)\n", filename,
 		list_print(pp_list);
 	*/
 
-	/* remember, we save filenames with escape sequences in the lists */
-	list_set(filename_escape(filename), t, howdeep, pp_list);
+	/*
+	 * Remember: we have escaped filenames (i.e. filenames with
+	 * funny characters replaced with \123 sequences) in cf and db
+	 * files, and maybe in some report messages (to avoid messing
+	 * up silly terminals that cannot cope with those characters;
+	 * why do people then have such files?), but internally we save
+	 * them in all their gory detail. Otherwise it would be too
+	 * hard to keep track of where to convert a name, or if it has
+	 * been converted already; in particular we screwed up on
+	 * saving the same way for list_set and list_set_flag.
+	 */
+	/*zzzzz Was: remember, we save filenames with escape sequences in the lists zzzzz*/
+	list_set(filename, t, howdeep, pp_list);
+	SPDEBUG(1000) printf("configfile_descend: adding %s\n", filename);
+
 	(void) list_setflag(filename, FLAG_NOOPEN, pp_list);
 
 	/* if it's a symbolic link, make sure we flag it as such! */
@@ -593,15 +632,18 @@ XENIX_CONT: ;
 	  case S_IFCHR:
 	  case S_IFBLK:
 #if !defined(SYSV) || (SYSV > 3)
+#ifndef apollo
+/* Foolish Apollos define S_IFSOCK same as S_IFIFO in /bsd4.3/usr/include/sys/stat.h */
 	  case S_IFSOCK:
+#endif
 #endif
 	    (void) list_setflag(filename, FLAG_NOOPEN, pp_list);
 	    break;
 #if !defined(SYSV) || (SYSV > 3)
 	  case S_IFLNK:	/* if it's a symbolic link, make sure we flag it as such! */
 	    (void) list_setflag(filename, FLAG_SYMLINK, pp_list);
-#endif
 	    break;
+#endif
 	  default:
 	    break;   /* do nothing for regular files */
 	}
