@@ -1,6 +1,5 @@
 #ifndef lint
-static char     rcsid[] = "$Id: ignorevec.c,v 1.9 1993/12/15 16:58:28 genek Exp $";
-
+static char     rcsid[] = "$Id: ignorevec.c,v 1.13 1994/07/15 11:03:44 gkim Exp $";
 #endif
 
 /*
@@ -70,7 +69,7 @@ ignore_configvec_to_dvec (s)
     char dvec[128];
     int  ignoremask = 0, ignorethis = 0;
     char *pc;
-    unsigned long l;
+    uint32 l;
 
     /*
      * where ignore-flags are in the format:
