@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "$Id: main.c,v 1.26 1993/12/15 17:35:29 genek Exp $";
+static char rcsid[] = "$Id: main.c,v 1.31 1994/08/26 08:23:03 gkim Exp $";
 #endif
 
 /************************************************************************
@@ -54,6 +54,7 @@ static char rcsid[] = "$Id: main.c,v 1.26 1993/12/15 17:35:29 genek Exp $";
 #ifdef MALLOCH
 # include <malloc.h>
 #endif
+#include <assert.h>
 #if (defined(SYSV) && (SYSV < 3))
 # include <limits.h>
 #endif	/* SVR2 */
@@ -130,6 +131,11 @@ char *specified_dbasefile = NULL;
 char *specified_configfile = NULL;
 int specified_configfd = -1;
 int specified_dbasefd = -1;
+
+/* if these vars are non-zero, specified_fd is guaranteed to be valid */
+int specified_dbasemode = 0;
+int specified_configmode = 0;
+
 int runtimeignore = 0;
 int interactivemode = 0;
 int test_interactive = 0;
@@ -185,7 +191,7 @@ version()
 {
     fprintf(stderr, "\nTripwire version %s (patchlevel %d)\n\n", version_num,
 			    PATCHLEVEL);
-    fprintf(stderr, "Copyright (c) 1992, 1993 Purdue Research Foundation\n");
+    fprintf(stderr, "Copyright (c) 1992, 1993, 1994 Purdue Research Foundation\n");
     fprintf(stderr, "\tBy Gene Kim, Eugene Spafford\n\n");
     exit(0);
 }
@@ -195,11 +201,10 @@ main(argc, argv)
     int argc;
     char *argv[];
 {
-    int i;
+    int i, fd;
     char *pc;
     char database[MAXPATHLEN+256];
     char mask[64];
-    int fd;
     char *specified_fd;
     int exitstatus = 0;
 
@@ -264,6 +269,7 @@ main(argc, argv)
 		usage();
 	    }
 	    specified_dbasefile = pc;
+	    specified_dbasemode |= SPECIFIED_FILE;
 	    continue;
 	}
 	/* specify configuration file */
@@ -273,6 +279,7 @@ main(argc, argv)
 		usage();
 	    }
 	    specified_configfile = pc;
+	    specified_configmode |= SPECIFIED_FILE;
 	    continue;
 	}
 	/* specify configuration file descriptor */
@@ -295,6 +302,7 @@ main(argc, argv)
 		perror(err);
 		exit(1);
 	    }
+	    specified_configmode |= SPECIFIED_FD;
 	    continue;
 	}
 	/* specify dbase file descriptor */
@@ -317,6 +325,7 @@ main(argc, argv)
 		perror(err);
 		exit(1);
 	    }
+	    specified_dbasemode |= SPECIFIED_FD;
 	    continue;
 	}
 	/* ignore specified signatures */
@@ -395,14 +404,27 @@ main(argc, argv)
     }
 
     /* argument sanity checking */
-    /*		are two files set to read from stdin? */
-    if (specified_configfile != NULL && specified_dbasefile) {
-	if (strcmp(specified_configfile, "-") == 0 &&
-	    strcmp(specified_dbasefile, "-") == 0) {
-	    fprintf(stderr, "%s: specified database and configuration file can't be both be stdin!\n", progname);
-	    exit(1);
-	}
+
+    /* eliminate aliases of stdin 
+     * 	(our canonical form is using fd 0)
+     */
+    if (specified_dbasefile && strcmp(specified_dbasefile, "-") == 0) {
+        specified_dbasefd = 0;
+	specified_dbasemode = SPECIFIED_FD;
+	specified_dbasefile = NULL;
     }
+    if (specified_configfile && strcmp(specified_configfile, "-") == 0) {
+        specified_configfd = 0;
+	specified_configmode = SPECIFIED_FD;
+	specified_configfile = NULL;
+    }
+
+    /* are two files set to read from stdin? */
+    if (specified_configfd == 0 && specified_dbasefd == 0) {
+	fprintf(stderr, "%s: specified database and configuration file can't be both be stdin!\n", progname);
+	exit(1);
+    }
+
     /* interactive mode and update mode? */
     if (interactivemode && pp_updateentries) {
         fprintf(stderr, "%s: conflicting mode directives!  Aborting...\n",
@@ -410,58 +432,55 @@ main(argc, argv)
 	exit(1);
     }
     /* specified configfile and configfd? */
-    if (specified_configfile && (specified_configfd != -1)) {
+    if ((specified_configmode & SPECIFIED_FILE) &&
+		(specified_configmode & SPECIFIED_FD)) {
 	fprintf(stderr, "%s: specified file and file descriptor for configuration file!\n", progname);
 	exit(1);
     }
     /* specified dbasefile and dbasefd? */
-    if (specified_dbasefile && (specified_dbasefd != -1)) {
+    if ((specified_dbasemode & SPECIFIED_FILE) &&
+		(specified_dbasemode & SPECIFIED_FD)) {
 	fprintf(stderr, "%s: specified file and file descriptor for database file!\n", progname);
 	exit(1);
     }
 
-    /*** we handle specified dbasefile file descriptors by writing it to
-     *** a file -- converting this case to a specified dbase file.
-     *** plus, it allows us to rewind the input stream properly
+    /* specified dbasefile and initialize dbase mode */
+    if (specified_dbasemode && dbaseinit) {
+	if (!quietmode) {
+	    fprintf(stderr, "%s: specifying a database file in database initialization mode \n", progname);
+	    fprintf(stderr, "\tis meaningless.  Ignoring specified file...\n");
+	}
+    }
+
+    /*** we check any specified file descriptors to make sure they are
+     *** files.  if not, we copy them into /tmp and return its fd.
+     *** (we unlink them as soon as we create them.)
      ***/
-    if (specified_dbasefd >= 0) {
-	char *tmpfilename;
-	FILE *fpin, *fpout;
-	int c;
 
-	/* generate temporary file name */
-	if ((tmpfilename = (char *) malloc(L_tmpnam)) == NULL) {
-	    perror("main: malloc()");
-	    exit(1);
-	};
-	(void) strcpy(tmpfilename, TEMPFILE_TEMPLATE);
+    switch(specified_dbasemode) {
+    case SPECIFIED_NONE:
+    	break;
+    case SPECIFIED_FILE:
+        specified_dbasefd = file_copy_to_tmp(specified_dbasefile);
+	break;
+    case SPECIFIED_FD:
+        specified_dbasefd = fd_copy_to_tmp(specified_dbasefd);
+	break;
+    default:
+        die_with_err("illegal specified_dbasemode state", NULL);
+    }
 
-	if ((char *) mktemp(tmpfilename) == NULL) {
-	    perror("main: mktemp()");
-	    exit(1);
-	}
-
-	/* copy file descriptor input into temporary file */
-	/*  input */
-	if (!(fpin = fdopen(specified_dbasefd, "r"))) {
-	    fprintf(stderr, "Couldn't open input database file descriptor!\n");
-	    exit(1);
-	}
-	/*  output */
-	if (!(fpout = fopen(tmpfilename, "w"))) {
-	    char err[1024];
-	    sprintf(err, "main: fopen(%s)", tmpfilename);
-	    perror(err);
-	    exit(1);
-	}
-	/*  copy */
-	while ((c = getc(fpin)) != EOF)
-	    putc(c, fpout);
-	fclose(fpin);
-	fclose(fpout);
-
-	/* save the name of the temporary file */
-	specified_dbasefile = tmpfilename;
+    switch(specified_configmode) {
+    case SPECIFIED_NONE:
+    	break;
+    case SPECIFIED_FILE:
+        specified_configfd = file_copy_to_tmp(specified_configfile);
+	break;
+    case SPECIFIED_FD:
+        specified_configfd = fd_copy_to_tmp(specified_configfd);
+	break;
+    default:
+        die_with_err("illegal specified_configmode state", NULL);
     }
 
     /* initialize lists */
@@ -484,7 +503,6 @@ main(argc, argv)
 #define DATABASE_REPOSITORY "./databases"
 
     if (dbaseinit || interactivemode || numupdateentries > 0) {
-	int fd;
 	if ((fd = open(DATABASE_REPOSITORY, 0)) >= 0) {
 	    close(fd);
 	} else {
@@ -538,13 +556,26 @@ main(argc, argv)
      * 		(this is meaningless if we specified stdin "-")
      */
 
-    if (specified_dbasefile == NULL)
-	sprintf(database, "%s/%s", database_path, database_file);
-    else
-	(void) strcpy(database, specified_dbasefile);
+    switch(specified_dbasemode) {
+    case SPECIFIED_NONE:
+	    sprintf(database, "%s/%s", database_path, database_file);
+	    break;
+    case SPECIFIED_FILE:
+	    (void) strcpy(database, specified_dbasefile);
+	    break;
+    case SPECIFIED_FD:
+	    break;
+    }
 
-    if (!printpreprocess && strcmp(database, "-") != 0) {
+    if (!printpreprocess && (specified_dbasemode != SPECIFIED_FD)) {
 	if ((fd = open(database, O_RDONLY)) < 0) {
+	    /* make sure our error message is correct */
+	    if (errno != ENOENT) {
+		char err[1024];
+		sprintf(err, "%s: database file `%s'", progname, database);
+		perror(err);
+		exit(1);
+	    }
 	    fprintf(stderr,
 		    "%s: database file '%s' does not exist!  Aborting...\n",
 		    progname, database);
@@ -575,7 +606,7 @@ main(argc, argv)
 	for (i = 0, pc = pp_updateentries[i]; pc; i++, 
 					    pc = pp_updateentries[i]) {
 SPDEBUG(0) 
-printf("Updating entry: %s\n", pc);
+printf("Updating entry: %s\n", filename_escape(pc));
 	}
 	numupdateentries = i;
 
@@ -584,6 +615,12 @@ printf("Updating entry: %s\n", pc);
 	}
 	update_mark(pp_updateentries, numupdateentries);
 	cleanup();
+	if (!quietmode) {
+	    fprintf(stderr, "###\n");
+	    fprintf(stderr, "### If you changed the tw.config file, remember to run `twdb_check.pl' to\n");
+	    fprintf(stderr, "### ensure database consistency.\n");
+	    fprintf(stderr, "### See the README file for details.\n");
+	}
 	exit(0);
     }
 
@@ -604,8 +641,10 @@ void
 cleanup()
 {
     /* delete temporary database file (derived from specified dbasefd) */
+    /*
     if (specified_dbasefd >= 0) {
 	unlink(specified_dbasefile);
     }
+    */
 }
 
