@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "$Id: nullsig.c,v 1.7 1993/12/02 18:03:23 genek Exp $";
+static char rcsid[] = "$Id: nullsig.c,v 1.12 1994/07/17 04:51:47 gkim Exp $";
 #endif
 
 /*
@@ -17,11 +17,20 @@ static char rcsid[] = "$Id: nullsig.c,v 1.7 1993/12/02 18:03:23 genek Exp $";
 #include <stdlib.h>
 #include <unistd.h>
 #endif
+#if !defined(SYSV) || (SYSV > 3)
+# include <sys/file.h>
+#else
+# include <unistd.h>
+#endif 	/* SYSV */
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "../include/tripwire.h"
 #include "../include/sigs.h"
 
+#ifndef SEEK_SET
+# define SEEK_SET L_SET
+#endif
+
 /*
  * int
  * pf_signature(int fd_in, char *ps_signature, int siglen)
