@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "$Id: preen.c,v 1.24 1993/12/15 16:50:20 genek Exp $";
+static char rcsid[] = "$Id: preen.c,v 1.29 1994/07/25 15:24:11 gkim Exp $";
 #endif
 
 /*
@@ -85,6 +85,7 @@ update_gather(interactive, ppp_updateentries)
 
     /* read in the old database */
     olddbasefile_load(&configentry_list);
+    assert(configentry_list);
 
     /* database_build() rewound the descriptor, so it's ready to use */
     fp_in = fptempdbase;
@@ -94,7 +95,8 @@ update_gather(interactive, ppp_updateentries)
 
     /* now build the report */
     preen_report(interactive, ppp_updateentries);
-    (void) fclose(fp_in);
+    if (!specified_configmode)
+	(void) fclose(fp_in);
 
     /* remove the temporary database file */
     (void) unlink(tempdatabase_file);
@@ -103,9 +105,6 @@ update_gather(interactive, ppp_updateentries)
 
     list_reset(&configentry_list);
 
-    free(configentry_list);
-
-
     return;
 }
 
@@ -158,7 +157,6 @@ list_print(&olddbase_list);
     /* iterate through the entries */
     for (i = 0; i < numentries; i++) {
 
-	char eentry[2048];		/* entry filename with escapes */
 	int isentry, isold, isnew;
 	int whichcase;
 
@@ -210,17 +208,15 @@ list_print(&olddbase_list);
 	 *		(inherit new ignore mask)
     	 */
 
-	strcpy(eentry, filename_escape(entry));
-
 	isentry = list_isthere(entry, &configentry_list);
 	isnew = list_isthere(entry, &newdbase_list);
-	isold = list_isthere(eentry, &olddbase_list);
+	isold = list_isthere(entry, &olddbase_list);
 
 	/* our case number is the vector of true/false bits */
 	whichcase = (isentry << 2) | (isold << 1) | (isnew);
 	if (!quietmode) { 
 	    fprintf(stderr, "Updating: %s: %s\n", updatemodes[whichcase], 
-		    entry); 
+		    filename_escape(entry));
 	}
 
 	switch(whichcase) {
@@ -234,14 +230,13 @@ list_print(&olddbase_list);
 	case UPDATE_ADDFILE:
 	{
 	    char entry_ignorevec[64]; 
-	    char number[64];
 	    char ignorestring[64];
 	    char *pc;
 	    int entrynum;
 	    char entryname[1024];
 
 	    if (verbosity)
-		fprintf(stderr, "Adding file %s\n", entry);
+		fprintf(stderr, "Adding file %s\n", filename_escape(entry));
 
 	    dbase_entry_howclose(entry, &configentry_list, entryname, 
 		&entrynum);
@@ -264,25 +259,25 @@ list_print(&olddbase_list);
 	    }
 
 	    sprintf(entry_ignorevec, "%d %s", entrynum, ignorestring);
-	    list_set(eentry, entry_ignorevec, 0, &olddbase_list);
-	    list_setflag(eentry, FLAG_UPDATE, &olddbase_list);
+	    list_set(entry, entry_ignorevec, MAXPATHLEN+1, &olddbase_list);
+	    list_setflag(entry, FLAG_UPDATE, &olddbase_list);
 
 	}
 	    break;
 	case UPDATE_DELETEFILE:
 	    if (verbosity)
-		fprintf(stderr, "Deleting file %s\n", entry);
-	    list_unset(eentry, &olddbase_list);
+		fprintf(stderr, "Deleting file %s\n", filename_escape(entry));
+	    list_unset(entry, &olddbase_list);
 	    break;
 	case UPDATE_UPDATEFILE:
 	{
 	    char oldignore[64], newignore[64], *pc, 
-			oldrest[1024], newrest[1024];
+			oldrest[1024];
 	    char newvalue[1024];
 	    int err, newentry, oldentry;
 
 	    if (verbosity)
-		fprintf(stderr, "Updating file %s\n", entry);
+		fprintf(stderr, "Updating file %s\n", filename_escape(entry));
 
 	    /* we know that the entry exists the the new dbaselist, otherwise,
 	     * we couldn't be in this case!
@@ -299,7 +294,7 @@ list_print(&olddbase_list);
 	    }
 
 	    /* parse the old dbase entry */
-	    pc = list_lookup(eentry, &olddbase_list);
+	    pc = list_lookup(entry, &olddbase_list);
 	    assert(pc != NULL);
 
 	    /* parse the old dbase entry */
@@ -311,7 +306,7 @@ list_print(&olddbase_list);
 		    break;
 		}
 		fprintf(stderr, "update_mark: olddbase_list parse error (nfields=%d, %s:%d)!\n", err, __FILE__, __LINE__);
-		fprintf(stderr, "%s>> %s\n", eentry, pc);
+		fprintf(stderr, "%s>> %s\n", entry, pc);
 		exit(1);
 	    }
 
@@ -325,21 +320,20 @@ list_print(&olddbase_list);
 	     */
 	    assert(strlen(newvalue) == strlen(pc));
 
-	    list_set(eentry, newvalue, 0, &olddbase_list);
-	    list_setflag(eentry, FLAG_UPDATE, &olddbase_list);
+	    list_set(entry, newvalue, MAXPATHLEN+1, &olddbase_list);
+	    list_setflag(entry, FLAG_UPDATE, &olddbase_list);
 	}
 	    break;
 	case UPDATE_ADDENTRY:
 	{
 	    struct list_elem *p;
-	    int entrynum;
 	    char ignorevec[128];
 	    int number;
 	    char *pc;
 	    int err;
 
 	    if (verbosity)
-		fprintf(stderr, "Adding entry %s\n", entry);
+		fprintf(stderr, "Adding entry %s\n", filename_escape(entry));
 
 	    if (!(pc = list_lookup(entry, &configentry_list))) {
 		fprintf(stderr, "%s: Can't find entry '%s'.  Skipping...\n",
@@ -355,10 +349,6 @@ list_print(&olddbase_list);
 	    list_open(&newdbase_list);
 	    while ((p = list_get(&newdbase_list))) {
 		int thisentry;
-		char escapedname[2048];
-
-		/* escape the name before inserting in olddbase_list */
-		strcpy(escapedname, filename_escape(p->varname));
 
 		if (sscanf(p->varvalue, "%d", &thisentry) != 1) {
 		    fprintf(stderr, "update_mark: newdbase_list parse error (nfields=%d, %s:%d)!\n", err, __FILE__, __LINE__);
@@ -369,8 +359,8 @@ list_print(&olddbase_list);
 		if (thisentry == number) {
 		    SPDEBUG(6) { printf("--(adding entry)--> %s\n", 
 			p->varname); }
-		    list_set(escapedname, p->varvalue, 0, &olddbase_list);
-		    list_setflag(escapedname, FLAG_UPDATE, &olddbase_list);
+		    list_set(p->varname, p->varvalue, MAXPATHLEN+1, &olddbase_list);
+		    list_setflag(p->varname, FLAG_UPDATE, &olddbase_list);
 		}
 	    }
 	    list_close(&olddbase_list);
@@ -381,10 +371,10 @@ list_print(&olddbase_list);
 	    if (verbosity) {
 		switch(whichcase) {
 		case UPDATE_DELETEENTRY:
-		    fprintf(stderr, "Deleting entry %s\n", entry);
+		    fprintf(stderr, "Deleting entry %s\n", filename_escape(entry));
 		    break;
 		case UPDATE_UPDATEENTRY:
-		    fprintf(stderr, "Updating entry %s\n", entry);
+		    fprintf(stderr, "Updating entry %s\n", filename_escape(entry));
 		break;
 		}
 	    }
@@ -410,18 +400,16 @@ list_print(&olddbase_list);
 	    }
 	    /* else we're deleting entries */
 	    else {
-		int flags;
-
 		dbase_entry_flag(&olddbase_list, entrynum, FLAG_DELETE, NULL);
 		SPDEBUG(20) list_print(&olddbase_list);
 
 		list_open(&olddbase_list);
-		while (p = list_get(&olddbase_list)) {
+		while ((p = list_get(&olddbase_list))) {
 		    if (q) {
 			list_unset(q->varname, &olddbase_list);
 			q = (struct list_elem *) NULL;
 		    }
-		    if ((flags = list_getflag(p->varname, &olddbase_list)) 
+		    if (list_getflag(p->varname, &olddbase_list)
 					& FLAG_DELETE) {
 			q = p;
 		    }
@@ -483,19 +471,26 @@ olddbasefile_load(ppentrylist)
 
     SPDEBUG(3) printf("*** entering olddbasefile_load()\n");
 
-    if (specified_dbasefile == NULL)
-	sprintf(filename, "%s/%s", database_path, database_file);
-    else
+    if (specified_dbasemode == SPECIFIED_FILE)
 	(void) strcpy(filename, specified_dbasefile);
+    else
+	sprintf(filename, "%s/%s", database_path, database_file);
 
-    /* check if we are to read from stdin */
-    if (strcmp(filename, "-") == 0) {
-	fp = stdin;
+    /* did we specify a dbase file? */
+    if (specified_dbasemode) {
+        if (!(fp = (FILE *) fdopen(specified_dbasefd, "r"))) {
+	    die_with_err("Couldn't open database file '%s'",
+			    filename);
+	}
+	rewind(fp);
+	if (ftell(fp) != 0) {
+	    die_with_err("olddbasefile_load: ftell()", NULL);
+	}
     }
     /* else open the file */
     else {
 	if ((fp = fopen(filename, "r")) == NULL)
-	    die_with_err("%s: Couldn't open database file '%s'",
+	    die_with_err("Couldn't open database file '%s'",
 			    filename);
     }
 
@@ -503,7 +498,6 @@ olddbasefile_load(ppentrylist)
     while (fgets(s, sizeof(s), fp) != NULL) {
 	char entryname[2048], *pc;
 	int oldentrynum, newentrynum;
-	char tmp[128];
 	static int countlines;
 
 	countlines++;
@@ -526,6 +520,7 @@ olddbasefile_load(ppentrylist)
 		__LINE__, s);
 	    exit(1);
 	}
+	filename_escape_expand(entryname);
 	if (!list_isthere(entryname, ppentrylist))
 	    continue;
 	pc = list_lookup(entryname, ppentrylist);
@@ -537,7 +532,7 @@ olddbasefile_load(ppentrylist)
 	    /* gotta do some chopping up of the contents information */
 	    sprintf(oldent, "%d", oldentrynum);
 	    sprintf(newent, "%d", newentrynum);
-	    list_set(oldent, newent, 0, &replace_list);
+	    list_set(oldent, newent, MAXPATHLEN+1, &replace_list);
 	}
     }
 
@@ -568,7 +563,6 @@ olddbasefile_load(ppentrylist)
 	/* skip database version */
 	else if (strcmp(key, "@@dbaseversion") == 0) {
 	    int version, err;
-	    char trash[1024];
 	    if ((err = sscanf(value, "%d", &version)) != 1) {
 		fprintf(stderr, "olddbasefile_load: @@dbaseversion parse error (nfields=%d, %s:%d)!\n", err, __FILE__, __LINE__);
 		    fprintf(stderr, ">> %s\n", key);
@@ -576,14 +570,26 @@ olddbasefile_load(ppentrylist)
 	    }
 
 	    if (version != DB_VERSION_NUM) {
-		fprintf(stderr,
-"error: %s: version error\n", filename);
-		fprintf(stderr,
-"\tdatabase format %d is no longer supported (expecting %d)!\n",
-			version, db_version_num);
-		fprintf(stderr,
+		/* special case */
+		if (DB_VERSION_NUM == 4 && version == 3) {
+		    fprintf(stderr,
+"%s: %s: version mismatch\n", progname, filename);
+		    fprintf(stderr,
+"\tdatabase format %d is no longer fully supported (expecting %d)!\n",
+			    version, db_version_num);
+		    fprintf(stderr,
 "\tSee tw.config(5) manual page for details\n");
-		exit(1);
+		}
+		else {
+		    fprintf(stderr,
+"%s: %s: version error\n", progname, filename);
+		    fprintf(stderr,
+"\tdatabase format %d is no longer supported (expecting %d)!\n",
+			    version, db_version_num);
+		    fprintf(stderr,
+"\tSee tw.config(5) manual page for details\n");
+		    exit(1);
+		}
 	    }
 	    continue;
 	}
@@ -597,6 +603,8 @@ olddbasefile_load(ppentrylist)
 SPDEBUG(10)
 printf("olddbasefile_load: %s: %s", key, value);
 
+	    filename_escape_expand(key);
+
 	    /* check to see if it's a special file */
 	    if ((err = sscanf(value, "%s %s %o %[^\n]", pcentry, pcignore, &mode,
 			pcrest)) != 4) {
@@ -608,16 +616,15 @@ printf("olddbasefile_load: %s: %s", key, value);
 
 	    /* do we need to replace the entry number? */
 	    if (list_isthere(pcentry, &replace_list)) {
-		char tmp[100];
 		/* splice the new value in */
 		sprintf(newvalue, "%s %s %o %s\n", 
 		    list_lookup(pcentry, &replace_list), 
 		    pcignore, mode, pcrest);
-		list_set(key, newvalue, 0, &olddbase_list);
+		list_set(key, newvalue, MAXPATHLEN+1, &olddbase_list);
 	    }
 	    /* otherwise the entry number was fine */
 	    else {
-		list_set(key, value, 0, &olddbase_list);
+		list_set(key, value, MAXPATHLEN+1, &olddbase_list);
 	    }
 
 	    switch (mode & S_IFMT) {
@@ -626,19 +633,26 @@ printf("olddbasefile_load: %s: %s", key, value);
 	      case S_IFDIR:
 	      case S_IFBLK:
 #if !defined(SYSV) || (SYSV > 3)
+#ifndef apollo
+/* Foolish Apollos define S_IFSOCK same as S_IFIFO in /bsd4.3/usr/include/sys/stat.h */
 	      case S_IFSOCK:
-	      case S_IFLNK:
+#endif
 #endif
 		(void) list_setflag(key, FLAG_NOOPEN, &olddbase_list);
+		break;
+#if !defined(SYSV) || (SYSV > 3)
+	      case S_IFLNK:	/* if it's a symbolic link, make sure we flag it as such! */
+		(void) list_setflag(key, FLAG_SYMLINK, &olddbase_list);
+		break;
+#endif
 	    }
 
 	}
     }
 
     /* close the file descriptor */
-    if (fp != stdin) {
+    if (!specified_dbasemode)
 	(void) fclose(fp);
-    }
 
     SPDEBUG(3) printf("*** leaving olddbasefile_load()\n");
 
@@ -670,7 +684,7 @@ dbase_entry_howclose(filename, ppentrylist, entry, pentrynum)
     int *pentrynum;
 {
     struct list_elem *p;
-    int maxscore = 0, maxentrynum;
+    int maxscore = 0;
 
     if (list_open(ppentrylist) < 0) {
 	fprintf(stderr, "%s: dbase_entry_howclose: list_open() failed!\n",
@@ -704,7 +718,6 @@ printf("dbase_entry_howclose: %d: (%s,%s)\n", score, filename, p->varname);
 
 	if (score > maxscore) {
 	    maxscore = score;
-	    maxentrynum = entrynum;
 	    strcpy(entry, p->varname);
 	    /* grab the entry number */
 	    if ((pold = list_lookup(p->varname, &olddbase_list))) {
