@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "$Id: preen.report.c,v 1.14 1993/12/02 18:03:25 genek Exp $";
+static char rcsid[] = "$Id: preen.report.c,v 1.20 1994/08/04 03:44:34 gkim Exp $";
 #endif
 
 /*
@@ -43,6 +43,7 @@ static char **pp_update = (char **) NULL;
 static void updateentry_prompt();
 static void updateentry_menu();
 static void updateentry_help();
+static int entrynum_get();
 
 /*
  * preen_report()
@@ -122,7 +123,6 @@ preen_report(interactive, ppp_updateentries)
 	char filename[2048];
 
 	strcpy(filename, p->varname);
-	filename_escape_expand(filename);
 
 	(void) structstat_fill(p->varvalue, &statnew, sigsnew, filename);
 
@@ -132,14 +132,16 @@ preen_report(interactive, ppp_updateentries)
 	 * list.  Abort if this happens.
 	 */
 	if (list_isthere(filename, &diff_deleted_list)) {
-	    printf("### Why is this file also marked as DELETED?  Please mail this output to (genek@cc.purdue.edu)!\n");
+	    printf("### Why is this file also marked as DELETED?  Please mail this output to (gkim@cs.purdue.edu)!\n");
 	}
 	if (list_isthere(filename, &diff_changed_list)) {
-	    printf("### Why is this file also marked as CHANGED?  Please mail this output to (genek@cc.purdue.edu)!\n");
+	    printf("### Why is this file also marked as CHANGED?  Please mail this output to (gkim@cs.purdue.edu)!\n");
 	}
 
 	if (interactive) {
-	    updateentry_prompt(fttyin, fttyout, filename, firsttime);
+	    int entrynum;
+	    entrynum = entrynum_get(p->varvalue);
+	    updateentry_prompt(fttyin, fttyout, filename, entrynum, firsttime);
 	    firsttime = 0;
 	}
     }
@@ -163,13 +165,14 @@ preen_report(interactive, ppp_updateentries)
 	char filename[2048];
 
 	strcpy(filename, p->varname);
-	filename_escape_expand(filename);
 
 	(void) structstat_fill(p->varvalue, &statnew, sigsnew, filename);
 
 	direntry_print(filename, statnew, DIFF_DELETED);
 	if (interactive) {
-	    updateentry_prompt(fttyin, fttyout, filename, firsttime);
+	    int entrynum;
+	    entrynum = entrynum_get(p->varvalue);
+	    updateentry_prompt(fttyin, fttyout, filename, entrynum, firsttime);
 	    firsttime = 0;
 	}
     }
@@ -199,12 +202,6 @@ preen_report(interactive, ppp_updateentries)
 
     /* print out each added file in sequence */
     while ((p = list_get(&diff_changed_list)) != NULL) {
-	char filename[2048];
-
-
-	/* copy the filename and expand any escaped characters */
-	strcpy(filename, p->varname);
-	filename_escape_expand(filename);
 
 	/* filename, ignore, mode, inode, nlinks, uid, gid, size, access,
 	 * modify, ctime, sig1, sig2
@@ -213,7 +210,7 @@ preen_report(interactive, ppp_updateentries)
 	/* read in the new value from the changed_list
 	 *		throw away the new ignorevector -- we use the old one!
 	 */
-	(void) structstat_fill(p->varvalue, &statnew, sigsnew, filename);
+	(void) structstat_fill(p->varvalue, &statnew, sigsnew, p->varname);
 
 	/* read in the list1 value form the hash table */
 	if ((s = list_lookup(p->varname, &olddbase_list)) == NULL) {
@@ -229,14 +226,14 @@ preen_report(interactive, ppp_updateentries)
 	    continue;
 	}
 
-	(void) structstat_fill(s, &statold, sigsold, filename);
+	(void) structstat_fill(s, &statold, sigsold, p->varname);
 
 	/* is this file to be ignored? */
 	if (!(list_getflag(p->varname, &diff_changed_list) & FLAG_CHANGED))
 	    continue;
 
 	/* print out the report for this file */
-	direntry_print(filename, statnew, DIFF_CHANGED);
+	direntry_print(p->varname, statnew, DIFF_CHANGED);
 
     }
 
@@ -309,10 +306,6 @@ printf("### =========== ============================= ==========================
     /* print out each added file in sequence */
     while ((p = list_get(&diff_changed_list)) != NULL) {
         static int firsttime = 1;
-	char filename[2048];
-
-	strcpy(filename, p->varname);
-	filename_escape_expand(filename);
 
 	/* filename, ignore, mode, inode, nlinks, uid, gid, size, access,
 	 * modify, ctime, sig1, sig2 .. sign
@@ -321,7 +314,7 @@ printf("### =========== ============================= ==========================
 	/* read in the list2 value from the changed_list
 	 *		throw away the new ignorevector -- we use the old one!
 	 */
-	(void) structstat_fill(p->varvalue, &statnew, sigsnew, filename);
+	(void) structstat_fill(p->varvalue, &statnew, sigsnew, p->varname);
 
 	/* read in the list1 value form the hash table */
 	if ((s = list_lookup(p->varname, &olddbase_list)) == NULL) {
@@ -329,7 +322,7 @@ printf("### =========== ============================= ==========================
 	    exit(1);
 	}
 
-	ignorevec = structstat_fill(s, &statold, sigsold, filename);
+	ignorevec = structstat_fill(s, &statold, sigsold, p->varname);
 
 	/* get the ignoremask */
 	ignoremask = ignore_vec_to_scalar(ignorevec);
@@ -338,7 +331,7 @@ printf("### =========== ============================= ==========================
 	if (!(list_getflag(p->varname, &diff_changed_list) & FLAG_CHANGED))
 	    continue;
 
-	printf("%s\n", filename);
+	printf("%s\n", p->varname);
 	/* and then the {expected, received} pairs */
 
 #define STATEQ(x) (statnew.x != statold.x)
@@ -350,38 +343,38 @@ printf("### =========== ============================= ==========================
 
 	if (!(ignoremask & IGNORE_P))
 	    if (STATEQ(st_mode)) {
-		pair_print_llo("st_mode:", (long) statnew.st_mode,
-			(long) statold.st_mode);
+		pair_print_llo("st_mode:", (int32) statnew.st_mode,
+			(int32) statold.st_mode);
 	    }
 	
 	if (!(ignoremask & IGNORE_I))
 	    if (STATEQ(st_ino)) {
-		pair_print_ll("st_ino:", (long) statnew.st_ino,
-			(long) statold.st_ino);
+		pair_print_ll("st_ino:", (int32) statnew.st_ino,
+			(int32) statold.st_ino);
 	    }
 	
 	if (!(ignoremask & IGNORE_N))
 	    if (STATEQ(st_nlink)) {
-		pair_print_ll("st_nlink:", (long) statnew.st_nlink,
-			(long) statold.st_nlink);
+		pair_print_ll("st_nlink:", (int32) statnew.st_nlink,
+			(int32) statold.st_nlink);
 	    }
 
 	if (!(ignoremask & IGNORE_U))
 	    if (STATEQ(st_uid)) {
-		pair_print_ll("st_uid:", (long) statnew.st_uid,
-			(long) statold.st_uid);
+		pair_print_ll("st_uid:", (int32) statnew.st_uid,
+			(int32) statold.st_uid);
 	    }
 
 	if (!(ignoremask & IGNORE_G))
 	    if (STATEQ(st_gid)) {
-		pair_print_ll("st_gid:", (long) statnew.st_gid,
-			(long) statold.st_gid);
+		pair_print_ll("st_gid:", (int32) statnew.st_gid,
+			(int32) statold.st_gid);
 	    }
 
 	if (!(ignoremask & IGNORE_S))
 	    if (STATEQ(st_size)) {
-		pair_print_ll("st_size:", (long) statnew.st_size,
-			(long) statold.st_size);
+		pair_print_ll("st_size:", (int32) statnew.st_size,
+			(int32) statold.st_size);
 	    }
 
 	if (!(ignoremask & IGNORE_A))
@@ -423,7 +416,9 @@ printf("### =========== ============================= ==========================
 
 	/* quiz the user if this entry should be updated */
 	if (interactive) {
-	    updateentry_prompt(fttyin, fttyout, filename, firsttime);
+	    int entrynum;
+	    entrynum = entrynum_get(p->varvalue);
+	    updateentry_prompt(fttyin, fttyout, p->varname, entrynum, firsttime);
 	    firsttime = 0;
 	}
 
@@ -457,7 +452,7 @@ preen_change_count()
     char *s;
     int ignoremask;
     char ignorevec[512];
-    unsigned long mode, ino, nlink, uid, gid, size;
+    uint32 mode, ino, nlink, uid, gid, size;
     int entrynum;
     int nfields;
 
@@ -480,12 +475,8 @@ preen_change_count()
     while ((p = list_get(&diff_changed_list)) != NULL) {
 
 	int isdir = 0;
-	char filename[2048];
 	int reallychanged = 0;
 
-	strcpy(filename, p->varname);
-	filename_escape_expand(filename);
-
 	/* filename, ignore, mode, inode, nlinks, uid, gid, size, access,
 	 * modify, ctime, sig1, sig2
 	 */
@@ -502,7 +493,7 @@ preen_change_count()
 		sigsnew[5], sigsnew[6], sigsnew[7], sigsnew[8], sigsnew[9]))
 				!= DB_RECORD_FIELDS) {
 	    fprintf(stderr, "preen_change_count: %s: illegal database record (nfields == %d).   Aborting...\n", 
-				 filename, nfields);
+				 p->varname, nfields);
 	    fprintf(stderr, "	'%s'\n", p->varvalue);
 	    exit(1);
 	}
@@ -535,7 +526,7 @@ preen_change_count()
 		sigsold[0], sigsold[1], sigsold[2], sigsold[3], sigsold[4],
 		sigsold[5], sigsold[6], sigsold[7], sigsold[8], sigsold[9]))
 				!= DB_RECORD_FIELDS) {
-	    fprintf(stderr, "preen_change_count: %s: illegal database record! Aborting...  (nfields=%d)\n", filename, nfields);
+	    fprintf(stderr, "preen_change_count: %s: illegal database record! Aborting...  (nfields=%d)\n", p->varname, nfields);
 	    fprintf(stderr, "	'%s'\n", s);
 	    exit(1);
 	}
@@ -556,7 +547,7 @@ preen_change_count()
 
 	/* and then the {expected, received} pairs */
 
-#define FLAGIT(x) changed++; reallychanged = 1; SPDEBUG(3) {printf("--(FLAGGING %s: unignored change in <%s>)--\n", filename, (x)); } list_setflag(p->varname, FLAG_CHANGED, &diff_changed_list); continue
+#define FLAGIT(x) changed++; reallychanged = 1; SPDEBUG(3) {printf("--(FLAGGING %s: unignored change in <%s>)--\n", p->varname, (x)); } list_setflag(p->varname, FLAG_CHANGED, &diff_changed_list); continue
 #define SIGEQ(x) if (strcmp(sigsnew[(x)], sigsold[(x)]) != 0)
 
 	/* allow for loose directory interpretations by ignoring nlink,
@@ -565,6 +556,18 @@ preen_change_count()
 	if (isdir && loosedir) {
 	    ignoremask |= IGNORE_N | IGNORE_M | IGNORE_C | IGNORE_S;
 	}
+#ifdef apollo
+	/*
+	 * Apollos do not keep ownership or dates for symlinks, but
+	 * get these from the directory containing them (with wide-open
+	 * permissions). Ignore these. (Why only if we also have loosedir?)
+	 * (Otherwise each symlink would get flagged whenever the directory
+	 * is changed, e.g. by adding or deleting a file.)
+	 */
+	else if (((mode & S_IFMT) == S_IFLNK) && loosedir) {
+	    ignoremask |= IGNORE_P | IGNORE_U | IGNORE_G | IGNORE_A | IGNORE_M | IGNORE_C;
+	}
+#endif
 
 	/* note the pain we go through to avoid dangling else's */
 	if (!(ignoremask & IGNORE_P)) { if (STATEQ(st_mode)) {FLAGIT("p");}}
@@ -630,7 +633,7 @@ structstat_fill (string, statbuf, sigs, recordkey)
 {
     char *ignorevec;
     static char structstat_fill_string[512];
-    unsigned long        mode, ino, nlink, uid, gid, size;
+    uint32        mode, ino, nlink, uid, gid, size;
     int entrynum;
     char vec64_a[50], vec64_m[50], vec64_c[50];
 
@@ -663,6 +666,28 @@ structstat_fill (string, statbuf, sigs, recordkey)
     return ignorevec;
 }
 
+/*
+ * entrynum_get(char *string)
+ *
+ *	given a string from the database, return the entrynum
+ */
+
+static int
+entrynum_get (string)
+    char *string;
+{
+    char trash[1024];
+    int entrynum;
+
+    if (sscanf(string, "%d %s", &entrynum, trash) != 2) {
+	fprintf(stderr, "entrynum_get: illegal database record!  Aborting...\n");
+	fprintf(stderr, ">>	'%s'\n", string);
+	exit(1);
+    }
+
+    return entrynum;
+}
+
 /*
  * pair_print_ss(char *label, char *s1, char *s2)
  *
@@ -680,32 +705,32 @@ pair_print_ss (label, s1, s2)
 }
 
 /*
- * pair_print_ll(char *label, long l1, long l2)
+ * pair_print_ll(char *label, int32 l1, int32 l2)
  *
- *	print {expected,received} table with longs
+ *	print {expected,received} table with int32s
  */
 
 static void
 pair_print_ll (label, l1, l2)
     char *label;
-    long l1;
-    long l2;
+    int32 l1;
+    int32 l2;
 {
     printf("%15s %-30ld%-30ld\n", label, l1, l2);
     return;
 }
 
 /*
- * pair_print_llo(char *label, long l1, long l2)
+ * pair_print_llo(char *label, int32 l1, int32 l2)
  *
- *	print {expected,received} table with longs in octal
+ *	print {expected,received} table with int32s in octal
  */
 
 static void
 pair_print_llo (label, l1, l2)
     char *label;
-    long l1;
-    long l2;
+    int32 l1;
+    int32 l2;
 {
     printf("%15s %-30lo%-30lo\n", label, l1, l2);
     return;
@@ -735,14 +760,19 @@ updateentry_list_add(filename)
 }
 
 static void
-updateentry_prompt(fttyin, fttyout, filename, reset)
+updateentry_prompt(fttyin, fttyout, filename, entrynum, reset)
     FILE *fttyin, *fttyout;
     char *filename;
+    int entrynum;
     int reset;
 {
     static int firsttime = 1;
     static int lastanswer = 0;
     char answer[100], *pc;
+    static int lastentrynum = -1;
+
+    SPDEBUG(100) printf("updateentry_prompt: (entrynum=%d, lastentrynum=%d)\n",
+    			entrynum, lastentrynum);
 
 #define LASTANSWER_ALLYES 	1
 #define LASTANSWER_ALLNO 	2
@@ -751,23 +781,36 @@ updateentry_prompt(fttyin, fttyout, filename, reset)
     if (reset) {
         firsttime = 1;
 	lastanswer = 0;
+	lastentrynum = -1;
     }
 
 AGAIN:
     if (lastanswer == LASTANSWER_ALLNO) {
-        return;
+	if (entrynum == lastentrynum) {
+	    return;
+	}
+	else {
+	    lastentrynum = -1;
+	    lastanswer = 0;
+	}
     }
     if (lastanswer == LASTANSWER_ALLYES || test_interactive) {
-	updateentry_list_add(filename); 
-	if (!test_interactive) {
-	    fprintf(fttyout, "---> Updating '%s'\n", filename);
-	    fflush(fttyout);
+	if (entrynum == lastentrynum) {
+	    updateentry_list_add(filename); 
+	    if (!test_interactive) {
+		fprintf(fttyout, "---> Updating '%s'\n", filename_escape(filename));
+		fflush(fttyout);
+	    }
+	    return;
+	} 
+	else {
+	    lastentrynum = -1;
+	    lastanswer = 0;
 	}
-	return;
     }
 
     if (!test_interactive) {
-	fprintf(fttyout, "---> File: '%s'\n", filename);
+	fprintf(fttyout, "---> File: '%s'\n", filename_escape(filename));
 	fprintf(fttyout, "---> Update entry?  [YN(y)nh?] ");
 	fflush(fttyout);
 	(void) fgets(answer, sizeof(answer), fttyin);
@@ -783,12 +826,15 @@ AGAIN:
     		break;
     case 'Y': 	updateentry_list_add(filename); 
     		lastanswer = LASTANSWER_ALLYES;
+		lastentrynum = entrynum;
 		break;
     case 'n':	break;
     case 'N': 	lastanswer = LASTANSWER_ALLNO;
+		lastentrynum = entrynum;
     		break;
     case 'h':   updateentry_help(fttyout);
-    		break;
+    		goto AGAIN;
+		break;
     case '?':	updateentry_menu(fttyout);
     		goto AGAIN;
     		break;
