@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "$Id: utils.c,v 1.14 1993/12/15 16:58:28 genek Exp $";
+static char rcsid[] = "$Id: utils.c,v 1.23 1994/07/25 16:23:16 gkim Exp $";
 #endif
 
 /*
@@ -15,8 +15,15 @@ static char rcsid[] = "$Id: utils.c,v 1.14 1993/12/15 16:58:28 genek Exp $";
 #include "../include/byteorder.h"
 #include <stdio.h>
 #include <assert.h>
+#include <fcntl.h>
+#if !defined(SYSV) || (SYSV > 3)
+# include <sys/file.h>
+#else
+# include <unistd.h>
+#endif 	/* SYSV */
 #ifdef STDLIBH
 #include <stdlib.h>
+#include <unistd.h>
 #endif
 #include <ctype.h>
 #ifdef STRINGH
@@ -35,8 +42,6 @@ static char rcsid[] = "$Id: utils.c,v 1.14 1993/12/15 16:58:28 genek Exp $";
 #include <pwd.h>
 #include <grp.h>
 #include <sys/param.h>
-#include <ctype.h>
-#include <sys/param.h>
 #ifndef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN 64
 #endif
@@ -54,6 +59,14 @@ static char rcsid[] = "$Id: utils.c,v 1.14 1993/12/15 16:58:28 genek Exp $";
 #include "../include/list.h"
 #include "../include/tripwire.h"
 
+#ifndef SEEK_SET
+# define SEEK_SET L_SET
+#endif
+
+#ifndef L_tmpnam
+# define L_tmpnam (unsigned int) MAXPATHLEN
+#endif
+
 static void print_perm();
 
 #ifndef S_IRGRP
@@ -227,7 +240,7 @@ string_split_ch (string, s, t, ch)
     char *string;
     char *s;
     char *t;
-    char ch;
+    int ch;
 {
     char *sp;
 
@@ -279,7 +292,7 @@ filename_escape_expand (filename)
     char *filename;
 {
     int i = 0;
-    char filetmp[MAXPATHLEN+256];
+    static char filetmp[MAXPATHLEN+256];
     int octal;
     register char *pcin = filename, *pcout = filetmp;
 
@@ -370,7 +383,7 @@ char *
 filename_escape (filename)
     char *filename;
 {
-    char filetmp[MAXPATHLEN+256];
+    static char filetmp[MAXPATHLEN+256];
     register char *pcin = filename, *pcout = filetmp;
     static char *octal_array[] = {
 	"000", "001", "002", "003", "004", "005", "006", "007",
@@ -398,16 +411,18 @@ filename_escape (filename)
 
     /* these must be replace everywhere in the filename */
     while (*pcin) {
-	if (isalnum(*pcin))
+	if (isalnum(*pcin)) {
 	    *pcout++ = *pcin;
-	else
+	}
+	else if (iscntrl(*pcin)) {
+	    *pcout++ = '\\';
+	    *pcout++ = *(pccopy = octal_array[(int)(*pcin)]);
+	    *pcout++ = *++pccopy;
+	    *pcout++ = *++pccopy;
+	} 
+	else {
 	    switch(*pcin) {
 	      case '\\':
-	      case '\n':
-	      case '\r':
-	      case '\t':
-	      case '\b':
-	      case '\f':
 	      case '\'':
 	      case '\"':
 	      case '@':
@@ -426,6 +441,7 @@ filename_escape (filename)
 		*pcout++ = *pcin;
 		break;
 	    }
+	}
 	pcin++;
     }
 
@@ -444,7 +460,7 @@ static char base64vec[] =
   "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
 #endif
 
-/* pltob64 -- walk through a vector of longs, convert them to 
+/* pltob64 -- walk through a vector of int32s, convert them to 
  *	network byte ordering, and then convert to base 64
  *
  *	this is the preferred interface to btob64.
@@ -453,24 +469,23 @@ static char base64vec[] =
 #define NUMTMPLONGS	1000
 char *
 pltob64(pl, pcout, numlongs)
-    unsigned long *pl;
+    uint32 *pl;
     char *pcout;
     int numlongs;
 {
     register int i;
-    register unsigned long *plto;
-    unsigned long larray[NUMTMPLONGS];
-    static int n = 0;
+    register uint32 *plto;
+    uint32 larray[NUMTMPLONGS];
 
     assert(numlongs < NUMTMPLONGS);
     /* we use our own ntohl() routines, but we have to do it in-place */
-    memcpy((char *) larray, (char *) pl, numlongs*sizeof(unsigned long));
+    memcpy((char *) larray, (char *) pl, numlongs*sizeof(uint32));
 
     for (i = 0, plto = larray; i < numlongs; i++) {
-	htonl(*plto++);
+	bs_htonl(*plto++);
     }
 
-    return btob64((unsigned char *) larray, (char *) pcout, numlongs*sizeof(unsigned long)*8);
+    return btob64((unsigned char *) larray, (char *) pcout, numlongs*sizeof(uint32)*8);
 }
 
 /* btob64  -- convert arbitrary bits to base 64 string
@@ -527,7 +542,7 @@ btob64(pcbitvec, pcout, numbits)
 #ifdef FOO
 char *
 ltob64(num, vec64)
-    register unsigned long num;
+    register uint32 num;
     char *vec64;
 {
     register char *p1 = vec64;
@@ -547,18 +562,18 @@ ltob64(num, vec64)
 #endif
 
 /*
- * long
+ * int32
  * b64toi(char *vec)
  *
- *	given a base-64 string, convert to a long.
+ *	given a base-64 string, convert to a int32.
  */
 
-long
+int32
 b64tol(vec)
     char *vec;
 {
     register char *pc;
-    register long num = 0L;
+    register int32 num = 0L;
 
 /* we use a different base-64 vector now to preseve zero's traditional
  * value.
@@ -604,12 +619,12 @@ b64tol(vec)
 #endif
 }
 
-long
+int32
 oldb64tol(vec)
     char *vec;
 {
     register char *pc;
-    register long num = 0L;
+    register int32 num = 0L;
 
     for (pc = vec; *pc; pc++) {
 	num <<= 6;
@@ -634,7 +649,7 @@ oldb64tol(vec)
  *	this routine was taken from crc_check.c, written by Jon Zeeff
  *	(zeeff@b-tech.ann-arbor.mi.us)
  *
- *	hacked for use in Tripwire by Gene Kim (genek@mentor.cc.purdue.edu).
+ *	hacked for use in Tripwire by Gene Kim.
  */
 
 void
@@ -680,7 +695,7 @@ direntry_print (name, statbuf, mode)
 	(void) strcpy(a_time, ctime(&statbuf.st_mtime));
 	a_time[24] = '\0';
 
-	print_perm((unsigned long)statbuf.st_mode);
+	print_perm((uint32)statbuf.st_mode);
 
 	(void) printf(" %-9.9s %7d %s", owner, statbuf.st_size,
 						a_time + 4);
@@ -692,12 +707,12 @@ direntry_print (name, statbuf, mode)
  *	This routine was taken from crc_check.c, written by Jon Zeeff
  *	(zeeff@b-tech.ann-arbor.mi.us)
  *
- *	hacked for use in Tripwire by Gene Kim (genek@mentor.cc.purdue.edu).
+ *	hacked for use in Tripwire by Gene Kim.
  */
 
 static void
 print_perm(perm)
-    unsigned long perm;
+    uint32 perm;
 {
 
 	char    string[20];
@@ -762,6 +777,93 @@ print_perm(perm)
 	(void) printf("%s", string);
 }
 
+/*
+ * generate a temporary filename, placing it into (s).  we assume that
+ * space is already allocated for (s).
+ */
+
+int
+fd_tempfilename_generate()
+{
+    char tmp[MAXPATHLEN+256];
+    int fd;
+
+    (void) strcpy(tmp, TEMPFILE_TEMPLATE);
+    if ((char *) mktemp(tmp) == NULL) {
+	perror("tempfilename_generate: mktemp()");
+	exit(1);
+    }
+
+    if ((fd = open(tmp, O_RDWR | O_CREAT, 0600)) < 0) {
+	perror("tempfilename_generate: open()");
+	exit(1);
+    }
+    /* unlink right away to make sure no one can tamper with our file */
+    unlink(tmp);
+
+    return fd;
+}
+
+/*
+ * read the entirety of input from file descriptor, copying into a file in /tmp.
+ * we unlink the file to prevent anyone from accessing it.
+ * we then return a file descriptor to that file.
+ */
+
+#define BSIZE  4096
+
+int
+fd_copy_to_tmp(fdin)
+    int fdin;
+{
+    int readin;
+    int fdout;
+    struct stat statbuf;
+    char buf[BSIZE];
+
+    /* we don't need to copy from the fd if it's a regular file */
+    if (fstat(fdin, &statbuf) < 0) {
+	die_with_err("fd_copy_to_tmp: fstat()", NULL);
+    }
+    if ((statbuf.st_mode & S_IFMT) == S_IFREG)
+        return fdin;
+
+    fdout = fd_tempfilename_generate();
+
+    while ((readin = read(fdin, buf, BSIZE)) != 0) {
+        if (readin < 0) {
+	    char *pc = (char *) malloc(100);
+	    sprintf(pc, "%d", fdin);
+	    die_with_err("fd_copy_to_tmp: read(%d)", pc);
+	    exit(1);
+	}
+	if (write(fdout, buf, readin) != readin) {
+	    die_with_err("fd_copy_to_tmp: write()", NULL);
+	}
+    }
+
+    close(fdin);
+    if (lseek(fdout, 0, SEEK_SET) != 0) {
+	die_with_err("fd_copy_to_tmp: lseek() rewind error!", NULL);
+    }
+
+    return fdout;
+}
+
+int
+file_copy_to_tmp(filename)
+    char *filename;
+{
+    int fdin, fdout;
+
+    if ((fdin = open(filename, O_RDONLY)) < 0) {
+        die_with_err("couldn't open %s", filename);
+    }
+
+    fdout = fd_copy_to_tmp(fdin);
+    return fdout;
+}
+
 #ifdef TEST
 int debuglevel;
 main() {
@@ -769,12 +871,12 @@ main() {
     char s[64][1024];
 
     for (i = 0; i < 64; i++) {
-	ltob64((long) i, s[i]);
+	ltob64((int32) i, s[i]);
 	printf("%d --> %s\n", i, s[i]);
     }
 
     for (i = 0; i < 64; i++) {
-	long l;
+	int32 l;
 	l = b64tol(s[i]);
 	printf("%ld\n", l);
 
