@@ -1,16 +1,24 @@
 #!/bin/sh
 
-# $Id: test1.sh,v 1.4 92/11/03 13:36:36 genek Exp $ 
+# $Id: test1.sh,v 1.12 1993/12/15 17:03:45 genek Exp $ 
+
+HOSTNAME=hostname
+# check to see if we ran from top-level makefile!
+if [ $# -ne 2 ]
+then 
+   echo "Sorry!  You must run this test from the top-level Makefile!"
+   exit 1
+fi
 
 HOSTNAME=$1
 DIST=$2
+ME=$0
 
-cat << 'GHK'
-
-===
+cat << GHK
+=== $ME: DESCRIPTION
 
     This shell script tests all the Tripwire signature routines.
-Consequently, this test may take awhile to complete, since even the 
+Consequently, this test may take awhile to complete, because even the
 slowest signature routines are exercised.  On a Sequent Symmetry
 running 16 Mhz Intel 80386s, this test takes over five minutes to
 complete.
@@ -23,7 +31,7 @@ machine-dependent routines are working correctly.
 
 GHK
 
-echo "=== test begin ==="
+echo "=== $ME: BEGIN ==="
 echo ''
 
 echo creating: ./tw.db_TEST.@
@@ -36,15 +44,15 @@ CURRPATH=`echo $CURRPATH | sed s,/tests$,,`
 sed s,/tmp/genek/$DIST,$CURRPATH, < ./tw.db_TEST > ./tw.db_TEST.@; 
 sed s,/tmp/genek/$DIST,$CURRPATH, < ./tw.conf.test > ./@tw.config; 
 
-../src/tripwire -c ./@tw.config -d ./tw.db_TEST.@; 
+../src/tripwire -loosedir -c ./@tw.config -d ./tw.db_TEST.@; 
 
-echo === test end === ;
+echo "=== $ME: END ===" 
 echo ''
 
 echo Tripwire should have only reported: 
 echo "    added:   $CURRPATH/tests/@tw.config" 
 echo "             $CURRPATH/tests/tw.db_TEST.@... "
-echo "             $CURRPATH/databases/... "
+echo "             $CURRPATH/tests/OKEXER... "
 echo "    changed: $CURRPATH/... (maybe some directory sizes...) "
 echo "             ...and any other files you may have changed!"
 echo ''
